@@ -18,9 +18,86 @@ import pandas as pd
 import scipy.special
 
 
-def rarefyBarcodes(barcodecounts, *,
-                   barcodecol='barcode', countcol='count',
-                   maxpoints=100000, logspace=True):
+def inverse_simpson_index(
+    barcodecounts,
+    *,
+    barcodecol="barcode",
+    countcol="count",
+    groupcols="library",
+):
+    """Inverse Simpson index (reciprocal probability two barcodes are same).
+
+    Parameters
+    ----------
+    barcodecounts: pandas.DataFrame
+        Data frame with barcode counts
+    barcodecol : str
+        Column in ``barcodecounts`` listing all unique barcodes.
+    countcol : str
+        Column in ``barcodecounts`` with counts of each barcode.
+    groupcols : str, list, or None
+        Columns in ``barcodecounts`` by which we group for calculations.
+
+    Returns
+    -------
+    pandas.DataFrame
+
+    Example
+    -------
+    >>> barcodecounts = pd.DataFrame.from_records(
+    ...        [('lib1', 'AA', 10),
+    ...         ('lib1', 'AT', 20),
+    ...         ('lib1', 'AC', 30),
+    ...         ('lib2', 'AA', 5)],
+    ...        columns=['library', 'barcode', 'count'])
+    >>> inverse_simpson_index(barcodecounts)
+      library  inverse_simpson_index
+    0    lib1               2.571429
+    1    lib2               1.000000
+
+    """
+    # based on here: https://gist.github.com/martinjc/f227b447791df8c90568
+    reserved_cols = ["dummy", "p2", "simpson_index", "inverse_simpson_index"]
+    for col in reserved_cols:
+        if col in barcodecounts.columns:
+            raise ValueError(f"`barcodecounts` cannot have column {col}")
+
+    if groupcols:
+        if isinstance(groupcols, str):
+            groupcols = [groupcols]
+    else:
+        groupcols = ["dummy"]
+        barcodecounts["dummy"] = "dummy"
+    req_cols = [barcodecol, countcol, *groupcols]
+    if not set(barcodecounts.columns).issuperset(req_cols):
+        raise ValueError(f"`barcodecounts` lacks columns {req_cols}")
+    if len(barcodecounts) != len(barcodecounts.groupby(req_cols)):
+        raise ValueError("`barcodecol` and `groupcols` not unique rows")
+
+    df = (
+        barcodecounts.assign(
+            p2=lambda x: (
+                x[countcol] / (x.groupby(groupcols)[countcol].transform("sum"))
+            )
+            ** 2
+        )
+        .groupby(groupcols, as_index=False)
+        .aggregate(simpson_index=pd.NamedAgg("p2", "sum"))
+        .assign(inverse_simpson_index=lambda x: 1 / x["simpson_index"])
+    )
+    if groupcols == ["dummy"]:
+        groupcols = []
+    return df[[*groupcols, "inverse_simpson_index"]]
+
+
+def rarefyBarcodes(
+    barcodecounts,
+    *,
+    barcodecol="barcode",
+    countcol="count",
+    maxpoints=100000,
+    logspace=True,
+):
     """Rarefaction curve of barcode observations.
 
     Note
@@ -30,7 +107,7 @@ def rarefyBarcodes(barcodecounts, *,
 
     Parameters
     ----------
-    barcodecounts :pandas.DataFrame
+    barcodecounts : pandas.DataFrame
         Data frame with counts to rarefy.
     barcodecol : str
         Column in `barcodecounts` listing all unique barcodes.
@@ -82,7 +159,7 @@ def rarefyBarcodes(barcodecounts, *,
 
     """
     if len(barcodecounts) != len(barcodecounts[barcodecol].unique()):
-        raise ValueError('non-unique barcodes in `barcodecounts`')
+        raise ValueError("non-unique barcodes in `barcodecounts`")
 
     # follow nomenclature at
     # https://en.wikipedia.org/wiki/Rarefaction_(ecology)#Derivation
@@ -100,26 +177,36 @@ def rarefyBarcodes(barcodecounts, *,
     nbarcodes = []
     lnFactorial_N = scipy.special.gammaln(N + 1)
     if logspace and N > maxpoints:
-        ncounts = list(numpy.unique(numpy.logspace(
-                       math.log10(1), math.log10(N),
-                       num=min(N, maxpoints)).astype('int')))
+        ncounts = list(
+            numpy.unique(
+                numpy.logspace(
+                    math.log10(1), math.log10(N), num=min(N, maxpoints)
+                ).astype("int")
+            )
+        )
     else:
-        ncounts = list(numpy.unique(numpy.linspace(
-                       1, N, num=min(N, maxpoints)).astype('int')))
+        ncounts = list(
+            numpy.unique(numpy.linspace(1, N, num=min(N, maxpoints)).astype("int"))
+        )
     for n in ncounts:
         lnFactorial_N_minus_n = scipy.special.gammaln(N - n + 1)
         i = numpy.nonzero(N - Nk - n >= 0)  # indices where this is true
         nbarcodes.append(
-                K - (num[i] * numpy.exp(
-                            scipy.special.gammaln(N - Nk[i] + 1) +
-                            lnFactorial_N_minus_n -
-                            lnFactorial_N -
-                            scipy.special.gammaln(N - Nk[i] - n + 1))
-                     ).sum()
+            K
+            - (
+                num[i]
+                * numpy.exp(
+                    scipy.special.gammaln(N - Nk[i] + 1)
+                    + lnFactorial_N_minus_n
+                    - lnFactorial_N
+                    - scipy.special.gammaln(N - Nk[i] - n + 1)
                 )
-    return pd.DataFrame({'ncounts': ncounts, 'nbarcodes': nbarcodes})
+            ).sum()
+        )
+    return pd.DataFrame({"ncounts": ncounts, "nbarcodes": nbarcodes})
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     import doctest
+
     doctest.testmod()
